# -*- coding: utf-8 -*-

"""The base Controller API."""

import tg
from tg import TGController, tmpl_context
from tg.render import render
from tg import request
from pylons.i18n import _, ungettext, N_
from tw.jquery import jquery_js

from libacr.lib import *

__all__ = ['Controller', 'BaseController']

class BaseController(TGController):
    """
    Base class for the controllers in the application.

    Your web application should have one of these. The root of
    your application is used to compute URLs used by your app.

    """

    def __call__(self, environ, start_response):
        """Invoke the Controller"""
        # TGController.__call__ dispatches to the Controller method
        # the request is routed to. This routing information is
        # available in environ['pylons.routes_dict']

        request.identity = request.environ.get('repoze.who.identity')
        tmpl_context.identity = request.identity
        jquery_js.inject()
        
        full_acr_js.inject()
        acr_css.inject()
        return TGController.__call__(self, environ, start_response)



from tg import expose, flash, require, request, redirect, tmpl_context, TGController, config, validate
from tg.controllers import WSGIAppController
import libacr

from libacr.controllers.admin.base import BaseAdminController
from libacr.plugins.base import AdminEntry, AcrPlugin, plugin_expose
from pylons.controllers.util import abort
import acr.lib.helpers
import acr.model
from repoze.what import predicates
from tw.api import WidgetsList
import tw.forms as twf

class StrollerPluginController(TGController):
    @expose('stroller.templates.index')
    def index(self, *args, **kw):
        self.check_for_enabled_stroller()
        from stroller.controllers.ecommerce import StrollerController
        return StrollerController().index(*args, **kw)

    @expose()
    def lookup(self, *args, **kw):
        self.check_for_enabled_stroller()
        from stroller.controllers.ecommerce import StrollerController
        return StrollerController(), list(args)

    def check_for_enabled_stroller(self):
        if not config.get('stroller_enabled', False):
            return abort(404, "eCommerce support not available")

        self.setup_stroller_db_on_first_request()

    def setup_stroller_db_on_first_request(self):
        from stroller.model.commerce import Product
        if not Product.__table__.exists(bind=acr.model.DBSession.bind):
            from stroller.model import setup_stroller_database
            from stroller.model import init_stroller_model

            init_stroller_model(acr.model.DBSession, acr.model.DeclarativeBase,
                                acr.model.User, acr.model.Group)
            acr.model.metadata.create_all(bind=acr.model.DBSession.bind)
            setup_stroller_database(acr.model.DBSession, acr.model.DeclarativeBase,
                                    acr.model.User, acr.model.Group)

class StrollerPlugin(AcrPlugin):
    uri = 'shop'

    @property
    def controller(self):
        config['stroller_root'] = self.plugin_url()
        return StrollerPluginController()

    def init_helpers(self):
        import stroller.helpers
        acr.lib.helpers.stroller_url = stroller.helpers.stroller_url
        acr.lib.helpers.icons.update(stroller.helpers.icons)
        acr.lib.helpers.category_icon = stroller.helpers.category_icon
 
    def init_model(self):
        from stroller.model import init_stroller_model
        Product, ProductInfo, Category, Order, OrderItem = init_stroller_model(acr.model.DBSession, acr.model.DeclarativeBase,
                                                                               acr.model.User, acr.model.Group)

    def setup_models(self, DBSession, DeclarativeBase, User, Group):
        from stroller.model import setup_stroller_database
        setup_stroller_database(DBSession, DeclarativeBase, User, Group)

    def __init__(self):
        try:
            import stroller
        except:
            config['stroller_enabled'] = False

        self.init_model()
        self.init_helpers()



"""Helpers to run multiple ACR sites inside the same WSGIDaemon and Process"""

from zope.interface import implements
from repoze.who.interfaces import IAuthenticator, IMetadataProvider
from sqlalchemy.orm.exc import NoResultFound, MultipleResultsFound

from sqlalchemy import engine_from_config
from tg import config, request
import threading

class AcrMultiSiteEngine(object):
    def __init__(self):
        self.engines = {}
        self.configs = {}
        self.tl = threading.local()

    @property
    def forced_script(self):
        try:
            return self.tl.forced_script
        except:
            return None

    def force_thread_env(self, environ):
        if environ:
            try:
                script_name = environ['SCRIPT_FILENAME']
            except:
                script_name = environ['HTTP_HOST']
        else:
            script_name = None
        self.tl.forced_script = script_name

    def __getattr__(self, name):
        if name in ('engines', 'forced_script', 'force_script', 'configs', 'tl'):
            return object.__getattribute__(self, name)

        config_key = None
        try:
            try:
                config_key = request.environ['SCRIPT_FILENAME']
            except:
                config_key = request.environ['HTTP_HOST']
        except:
            pass

        if not self.forced_script and config_key and not self.configs.has_key(config_key):
            self.configs[config_key] = config['sqlalchemy.url']

        try:
            used_db = self.configs[config_key]
        except KeyError:
            used_db = config['sqlalchemy.url']

        if self.forced_script:
            try:
                used_db = self.configs[self.forced_script]
            except KeyError:
                pass

        if not self.forced_script and not config_key:
            raise Exception('Outside of a request and no script forced')

        if not self.engines.has_key(used_db):
            self.engines[used_db] = engine_from_config(config, 'sqlalchemy.')

        return getattr(self.engines[used_db], name)

class AcrAuthenticatorPlugin(object):
    implements(IAuthenticator, IMetadataProvider)

    def __init__(self, user_class, dbsession):
        self.user_class = user_class
        self.dbsession = dbsession

    def get_user(self, environ, username):
        from acr import model
        username_attr = getattr(self.user_class, 'user_name')

        query = self.dbsession.query(self.user_class)
        query = query.filter(username_attr==username)

        try:
            model.multisite_engine.force_thread_env(environ)
            r = query.one()
            model.multisite_engine.force_thread_env(None)
            return r
        except:
            # As recommended in the docs for repoze.who, it's important to
            # verify that there's only _one_ matching userid.
            return None

    def get_groups(self, environ, user):
        from acr import model
        model.multisite_engine.force_thread_env(environ)
        groups = [g.group_name.encode('utf-8') for g in user.groups]
        model.multisite_engine.force_thread_env(None)
        return groups

    # IAuthenticator
    def authenticate(self, environ, identity):
        if not ('login' in identity and 'password' in identity):
            return None

        user = self.get_user(environ, identity['login'])
        if user:
            validator = getattr(user, 'validate_password')
            if validator(identity['password']):
                return identity['login']

    # IMetadataProdiver
    def add_metadata(self, environ, identity):
        identity['user'] = self.get_user(environ, identity['repoze.who.userid'])
        identity['groups'] = self.get_groups(environ, identity['user'])

        environ['repoze.what.credentials'] = {}
        environ['repoze.what.credentials']['groups'] = identity['groups']


# -*- coding: utf-8 -*-

"""The application's Globals object"""

__all__ = ['Globals']
from libacr.views.manager import ViewsManager
from libacr.plugins.manager import PluginsManager
from libacr.mediaWorker import start_media_worker

import tg
if tg.version == '2.1':
    print 'TG2.1 found, helpers work-around in place'
    import tg.render, pylons
    def patched_pylons_globals():
        x = tg.render.my_pylons_globals()
        if x['h'] == {}:
            conf = pylons.config._current_obj()
            x['h'] = conf['package'].lib.helpers
        return x
    pylons.templating.pylons_globals = patched_pylons_globals

class Globals(object):
    """Container for objects available throughout the life of the application.

    One instance of Globals is created during application initialization and
    is available during requests via the 'app_globals' variable.

    """

    def __init__(self):
        self.acr_viewmanager = ViewsManager()
        self.plugins = PluginsManager()
        """Do nothing, by default."""
        from turbomail.adapters import tm_pylons
        tm_pylons.start_extension()



# -*- coding: utf-8 -*-

"""WebHelpers used in acr."""
from webhelpers import date, feedgenerator, html, number, misc, text
import libacr
from libacr.helpers import *

icons = {}
icons.update(libacr.helpers.icons)



# -*- coding: utf-8 -*-
"""Setup the acr application"""

import logging

import transaction
from tg import config

from acr.config.environment import load_environment
from libacr.model.user_permission import AcrUserPermission
from libacr.plugins.manager import PluginsManager

__all__ = ['setup_app']

log = logging.getLogger(__name__)

def create_content(name, data):
    from acr import model
    
    cnt = model.Content(name=name)
    model.DBSession.add(cnt)
    
    cnt_data = model.ContentData(content=cnt, revision=0, value=data)
    model.DBSession.add(cnt_data)
    
    return cnt

def setup_app(command, conf, vars):
    """Place any commands to setup acr here"""
    load_environment(conf.global_conf, conf.local_conf)
    # Load the models
    from acr import model
    model.multisite_engine.force_thread_env({'SCRIPT_FILENAME':'setup-app'})

    print "Creating tables"
    model.metadata.create_all(bind=config['pylons.app_globals'].sa_engine)
    print "Creating Data"

    manager = model.User()
    manager.user_name = u'manager'
    manager.display_name = u'Example manager'
    manager.email_address = u'manager@somedomain.com'
    manager.password = u'managepass'
    model.DBSession.add(manager)
    
    editor = model.User()
    editor.user_name = u'editor'
    editor.display_name = u'Example editor'
    editor.email_address = u'editor@somedomain.com'
    editor.password = u'editpass'
    model.DBSession.add(editor)

    group = model.Group()
    group.group_name = u'managers'
    group.display_name = u'Managers Group'
    group.users.append(manager)
    model.DBSession.add(group)
    
    group = model.Group()
    group.group_name = u'editors'
    group.display_name = u'Editors Group'
    group.users.append(manager)
    group.users.append(editor)
    model.DBSession.add(group)
    
    group = model.Group()
    group.group_name = u'acr'
    group.display_name = u'Acr Group'
    group.users.append(manager)
    model.DBSession.add(group)
  
    user_permission = AcrUserPermission()
    user_permission.user=manager
    user_permission.page="all"
    user_permission.can_edit=1
    user_permission.can_create_children=1
    model.DBSession.add(user_permission)

    t = model.Tag(name='story')
    model.DBSession.add(t)
  
    default_page = model.Page(uri='default', title='Default Page for global layout')
    model.DBSession.add(default_page)
  
    p = model.Page(uri='index', title='Index')
    model.DBSession.add(p)

    c = create_content(name='acr_presentation', data=u"""<h1>Welcome to <em>ACR</em></h1>
<img class="left" width="120" height="120" src="/acr/plugins/theme/statics/dragonfly/images/img07.jpg"/>
<p><strong>ACR</strong> is an opensource CMS based on <a href="http://www.turbogears.org">TurboGears2</a> developed by <a href="http://www.axant.it">AXANT</a>
feel free to use it for any project and for more informations refer to <a href="http://labs.axant.it/acr">labs.axant.it</a>. <em>Enjoy ;)</em></p>
<div class="clearfix"></div>
""")
    ci = model.Slice(view='html', content=c, page=p, zone='main', name="acr_presentation")
    model.DBSession.add(ci)

    c = create_content(name='acr_index_part2', data=u"""<h1>Welcome to your page!</h1>
<p>You are now ready to use ACR, this is your index page and in the menu there is already a link to the ACR page itself,
<a href="/login">login</a> and start creating new nodes for your site</p>
""")
    ci = model.Slice(view='html', content=c, page=p, zone='main', name="acr_index_part2")
    model.DBSession.add(ci)

    c = create_content(name='acr_home', data="http://labs.axant.it/acr")
    p = model.Page(uri='acr', title='ACR Project')
    model.DBSession.add(p)
    ci = model.Slice(view='link', content=c, page=p, zone='main', name="acr_home")
    model.DBSession.add(ci)    
      
    c = create_content(name="default_menu", data="[menu]\ndepth=1\nalign=horizontal\nroot=root")
    ci = model.Slice(view='menu', content=c, page=default_page, zone='header', name="menu")
    model.DBSession.add(ci)
    
    c = create_content(name="default_header", data=u"""
<h1><a href="#">ACR</a></h1>
<h2><a href="http://labs.axant.it/acr">Advanced Content Repository</a></h2>
""")
    ci = model.Slice(view='html', content=c, page=default_page, zone='header', name="header")
    model.DBSession.add(ci)
        
    c = create_content(name="default_footer", data=u"""<p>Powered by ACR</p>
<p>Copyright © 2009 AXANT. All Rights Reserved</p>""")
    ci = model.Slice(view='html', content=c, page=default_page, zone='footer', name="footer")
    model.DBSession.add(ci)
    
    c = create_content(name="default_sidebar", data=u"""<div class="orangebox">
<h2>Links</h2>
<ul>
    <li><a href="http://labs.axant.it/acr">ACR</a></li>
    <li><a href="http://www.axant.it">AXANT</a></li>
    <li><a href="http://www.turbogears.org">TurboGears</a></li>
</ul>
</div>
""")
    ci = model.Slice(view='html', content=c, page=default_page, zone='rightbar', name="sidebar")
    model.DBSession.add(ci)

    model.DBSession.flush()
    transaction.commit()

    PluginsManager.setup_models(model.DBSession, model.DeclarativeBase,
                                model.User, model.Group)

    model.DBSession.flush()
    transaction.commit()
    print "Successfully setup"



from datetime import datetime
from tg import expose, flash, require, request, redirect, tmpl_context, TGController, config, validate
from tg.controllers import WSGIAppController
import libacr
from libacr.controllers.admin.base import _create_node
from libacr.plugins.base import AdminEntry, AcrPlugin, plugin_expose
from libacr.controllers.admin.base import _create_node, BaseAdminController
from libacr.model.core import DBSession
from libacr.model.content import Tag, Page, Slice, Content, ContentData
from libacr.lib import url, current_user_id, language, icons, user_can_modify, get_page_from_urllist
from repoze.what import predicates
from tw.api import WidgetsList
import tw.forms as twf
from paste.urlparser import StaticURLParser
import os, shutil, zipfile, tempfile
from libacr.views.manager import ViewsManager

class ThemeStaticsController(TGController):
    @expose()
    def lookup(self, theme, *args):
        site_dir = os.path.join(config.get('public_dir'), 'themes', theme)
        return WSGIAppController(StaticURLParser(site_dir)), args

class ThemeController(BaseAdminController):
    statics = ThemeStaticsController()

    @plugin_expose('edit_css')
    @require(predicates.in_group("acr"))
    def edit_css(self):
        class EditCssForm(WidgetsList):
            css = twf.TextArea(suppress_label=True)
        css_form = twf.TableForm(fields=EditCssForm(), submit_text="Save")

        active_theme = Themable.get_theme_data()
        current_css = {}
        current_css['css'] = open(os.path.join(config.get('public_dir'), 'themes',
                                  active_theme.get_property('theme', 'active'), 'main.css')).read().decode('utf-8')
        return dict(form=css_form, current_css=current_css)

    @expose()
    @require(predicates.in_group("acr"))
    def save_css(self, css):
        active_theme = Themable.get_theme_data()
        css_file = open(os.path.join(config.get('public_dir'), 'themes',
                        active_theme.get_property('theme', 'active'), 'main.css'), 'w').write(css.encode('utf-8'))

        flash('CSS Successfully updated')
        return redirect(url('/plugins/theme/edit_css'))

    @plugin_expose('manage_themes')
    @require(predicates.in_group("acr"))
    def manage_themes(self):
        themes_path = os.path.join(config.get('public_dir'), 'themes')
        available_themes = os.listdir(themes_path)
        active_theme = Themable.get_theme_data().get_property('theme', 'active')
        return dict(available_themes=available_themes, active_theme=active_theme)

    @expose()
    @require(predicates.in_group("acr"))
    def delete(self, theme):
        active_theme = Themable.get_theme_data().get_property('theme', 'active')
        if theme == active_theme:
            flash('Cannot delete active theme')
            return redirect(url('/plugins/theme/manage_themes'))
        else:
            theme_path = os.path.join(config.get('public_dir'), 'themes', theme)
            shutil.rmtree(theme_path)
            flash('Theme deleted')
            return redirect(url('/plugins/theme/manage_themes'))

    @expose()
    @require(predicates.in_group("acr"))
    def clone(self, theme):
        old_theme_path = os.path.join(config.get('public_dir'), 'themes', theme)
        new_theme = theme + str(len(os.listdir(os.path.join(config.get('public_dir'), 'themes'))))
        new_theme_path = os.path.join(config.get('public_dir'), 'themes', new_theme)

        if os.path.exists(new_theme_path):
            flash('A theme with that name already exists')
            return redirect(url('/plugins/theme/manage_themes'))

        if not os.path.exists(old_theme_path):
            flash('Origin theme not found')
            return redirect(url('/plugins/theme/manage_themes'))

        shutil.copytree(old_theme_path, new_theme_path)
        flash('Theme copied')
        return redirect(url('/plugins/theme/manage_themes'))

    @expose()
    @require(predicates.in_group("acr"))
    def switch(self, theme, setup=False, clear=False):
        theme_path = os.path.join(config.get('public_dir'), 'themes', theme)
        if not os.path.exists(theme_path):
            flash('Theme does not exists')
            return redirect(url('/plugins/theme/manage_themes'))

        if setup and os.path.exists(os.path.join(theme_path, 'setup.acr')):
            if clear:
                for slice in DBSession.query(Slice):
                    DBSession.delete(slice)
            execute_acri(open(os.path.join(theme_path, 'setup.acr')).read())

        Themable.get_theme_data().set_property('theme', 'active', theme)
        flash('Theme enabled')
        return redirect(url('/plugins/theme/manage_themes'))

    @plugin_expose('upload_theme')
    @require(predicates.in_group("acr"))
    def upload_theme(self, **kw):
        class ThemeUploadForm(WidgetsList):
            themezip = twf.FileField(suppress_label=True)
        upload_form = twf.TableForm(fields=ThemeUploadForm(), submit_text="Upload")
        return dict(form=upload_form)

    @expose()
    @require(predicates.in_group("acr"))
    @validate(validators=dict(themezip=twf.validators.FieldStorageUploadConverter(not_empty=True)), error_handler=upload_theme)
    def do_upload(self, themezip):
        tfile = tempfile.NamedTemporaryFile('rw+b')
        tfile.write(themezip.file.read())
        tfile.flush()

        if not zipfile.is_zipfile(tfile.name):
            flash('Not a valid zip file')
            return redirect(url('/admin'))

        themezip = zipfile.ZipFile(tfile.name, 'r')
        theme_entries = themezip.namelist()
        if theme_entries[0].startswith('..') or theme_entries[0].startswith('/'):
            flash('Theme zip file cannot contain relative paths')
            return redirect(url('/admin'))

        for entry in theme_entries:
            if not entry.startswith(theme_entries[0]):
                flash('Theme zip file must contain all the theme files inside a root directory with theme name')
                return redirect(url('/admin'))

        themes_dir = os.path.join(config.get('public_dir'), 'themes')
        if os.path.exists(os.path.join(themes_dir, theme_entries[0])):
            flash('Theme already exists')
            return redirect(url('/plugins/theme/manage_themes'))

        for entry in theme_entries:
            data = themezip.read(entry)
            path = os.path.join(themes_dir, entry)
            if not data:
                os.mkdir(path)
            else:
                f = open(path, 'w')
                f.write(data)
                f.close()
        flash('Theme extracted')
        return redirect(url('/plugins/theme/manage_themes'))

def execute_acri(script):
    def create_tag(tname):
        if not DBSession.query(Tag).filter_by(name=tname).first():
            t = Tag(name=tname)
            DBSession.add(t)

    def create_slice(parent, name, view='html', data={'data':''}):
        page = get_page_from_urllist(parent.split('/'))

        if not DBSession.query(Slice).filter_by(name=name).first():
            slice_data = {}
            slice_data['name'] = name
            slice_data['page'] = page.uid
            slice_data['zone'] = 'main'
            slice_data['order'] = 0
            slice_data['tags'] = []
            slice_data['view'] = view
            slice_data['data'] = ViewsManager.encode(view, data)
            _create_node(**slice_data)

    def add_slice_tag(slice_name, tag_name):
        sli = DBSession.query(Slice).filter_by(name=slice_name).one()
        t = DBSession.query(Tag).filter_by(name=tag_name).one()
        sli.tags.append(t)

    def attach_javascript(pname, filename):
        page = ''
        if not pname:
            page = DBSession.query(Page).filter_by(uri='default').one()
        else:
            page = DBSession.query(Page).filter_by(name=pname).one()

        name = strip(filename, punctuation) # check
        js = ''
        if os.path.exists(os.path.join(theme_path, filename)):
            js = open(os.path.join(theme_path, filename)).read()
        #open file and read it

        if js and not DBSession.query(Slice).filter_by(name=name).first():
            slice_data = {}
            slice_data['name'] = name
            slice_data['page'] = page.uid
            slice_data['zone'] = 'main'
            slice_data['order'] = 0
            slice_data['tags'] = []
            slice_data['view'] = 'script'
            slice_data['data'] = js #file content
            _create_node(**slice_data)

    def create_page(uri, parent=None):
        parent_page = None
        if parent:
            parent_page = get_page_from_urllist(parent.split('/'))

        page = DBSession.query(Page).filter_by(uri=uri).first()
        if not page and ((parent and parent_page) or (not parent)):
            page = Page(uri=uri, title='Unknown', parent=parent_page)
            DBSession.add(page)

    def set_page_title(uri, *args):
        page = get_page_from_urllist(uri.split('/'))
        if page:
            page.title = ' '.join(args)

    def set_slice_zone(slice_name, zone):
        sli = DBSession.query(Slice).filter_by(name=slice_name).one()
        sli.zone = zone

    commands = {'create_tag':create_tag,
                'create_slice':create_slice,
                'add_slice_tag':add_slice_tag,
                'create_page':create_page,
                'set_page_title':set_page_title,
                'set_slice_zone':set_slice_zone,
                'attach_javascript':attach_javascript}

    for line in script.split('\n'):
        params = line.split()

        if params:
            cmd = params[0]
            params = params[1:]
            commands[cmd](*params)

class Themable(AcrPlugin):
    uri = 'theme'

    @staticmethod
    def get_theme_data():
        config = DBSession.query(Content).filter_by(name='_acr_config_theme').first()
        if not config:
            config = Content(name='_acr_config_theme')
            cdata = ContentData(content=config, revision=0)
            cdata.set_property('theme', 'active', 'dragonfly')
            DBSession.add(config)
            DBSession.add(cdata)
        return config.get_data_instance_for_lang()

    def __init__(self):
        self.admin_entries = [AdminEntry(self, 'Edit Css', 'edit_css', icon='edit_css.png', section="Themes"),
                              AdminEntry(self, 'Manage Themes', 'manage_themes', icon='switch_theme.png', section="Themes"),
                              AdminEntry(self, 'Upload Theme', 'upload_theme', icon='upload_theme.png', section="Themes")]
        self.controller = ThemeController()

    def theme_url(self, what):
        config = Themable.get_theme_data()
        what = 'statics/%s/%s' % (config.get_property('theme', 'active'), what)
        return self.plugin_url(what)



# -*- coding: utf-8 -*-
"""
Global configuration file for TG2-specific settings in acr.

This file complements development/deployment.ini.

Please note that **all the argument values are strings**. If you want to
convert them into boolean, for example, you should use the
:func:`paste.deploy.converters.asbool` function, as in::
    
    from paste.deploy.converters import asbool
    setting = asbool(global_conf.get('the_setting'))
 
"""

from tg.configuration import AppConfig

import acr
from acr import model
from acr.lib import app_globals, helpers

from acr.lib.multisite import AcrAuthenticatorPlugin
from repoze.who.middleware import PluggableAuthenticationMiddleware
from repoze.who.plugins.friendlyform import FriendlyFormPlugin
from repoze.who.plugins.auth_tkt import AuthTktCookiePlugin

class AcrCmsMultiSiteConfig(AppConfig):
	def add_auth_middleware(self, app, skip_authentication):
	    form_plugin = FriendlyFormPlugin('/login', '/login_handler', '/post_login', '/logout_handler', '/post_logout',
				      login_counter_name=None,
				      rememberer_name='cookie')

	    challengers = [('form', form_plugin)]
	    authenticators = [('acr_auth', AcrAuthenticatorPlugin(model.User, model.DBSession))]
	    mdproviders = [('acr_meta', AcrAuthenticatorPlugin(model.User, model.DBSession))]
		  
	    cookie = AuthTktCookiePlugin(self.sa_auth.cookie_secret, 'acrauth')
	    identifiers = [('cookie', cookie), ('form', form_plugin)]

	    from repoze.who.classifiers import default_request_classifier
	    from repoze.who.classifiers import default_challenge_decider
	    
	    app = PluggableAuthenticationMiddleware(app,
						  identifiers,
						  authenticators,
						  challengers,
						  mdproviders,
						  default_request_classifier,
						  default_challenge_decider)
	 
	    return app

base_config = AcrCmsMultiSiteConfig()
base_config.renderers = []

base_config.package = acr

#Set the default renderer
base_config.default_renderer = 'genshi'
base_config.renderers.append('genshi')

#NEEDED WHEN USING TG2.1
base_config.renderers.append('json')
base_config.sa_auth.cookie_secret = 'mycookiesecret'

# if you want raw speed and have installed chameleon.genshi
# you should try to use this renderer instead.
# warning: for the moment chameleon does not handle i18n translations
#base_config.renderers.append('chameleon_genshi')

#Configure the base SQLALchemy Setup
base_config.use_sqlalchemy = True
base_config.model = acr.model
base_config.DBSession = acr.model.DBSession

# Configure the authentication backend
base_config.auth_backend = 'sqlalchemy'
base_config.sa_auth.dbsession = model.DBSession
# what is the class you want to use to search for users in the database
base_config.sa_auth.user_class = model.User
# what is the class you want to use to search for groups in the database
base_config.sa_auth.group_class = model.Group
# what is the class you want to use to search for permissions in the database
base_config.sa_auth.permission_class = model.Permission

# override this if you would like to provide a different who plugin for
# managing login and logout of your application
base_config.sa_auth.form_plugin = None

# You may optionally define a page where you want users to be redirected to
# on login:
base_config.sa_auth.post_login_url = '/post_login'

# You may optionally define a page where you want users to be redirected to
# on logout:
base_config.sa_auth.post_logout_url = '/post_logout'



# -*- coding: utf-8 -*-
"""
Auth* related model.

This is where the models used by :mod:`repoze.who` and :mod:`repoze.what` are
defined.

It's perfectly fine to re-use this definition in the acr application,
though.

"""
import os
from datetime import datetime
import sys
try:
    from hashlib import sha1
except ImportError:
    sys.exit('ImportError: No module named hashlib\n'
             'If you are on python2.4 this library is not part of python. '
             'Please install it. Example: easy_install hashlib')

from sqlalchemy import Table, ForeignKey, Column
from sqlalchemy.types import Unicode, Integer, DateTime
from sqlalchemy.orm import relation, synonym

from acr.model import DeclarativeBase, metadata, DBSession

__all__ = ['User', 'Group', 'Permission']


#{ Association tables


# This is the association table for the many-to-many relationship between
# groups and permissions. This is required by repoze.what.
group_permission_table = Table('tg_group_permission', metadata,
    Column('group_id', Integer, ForeignKey('tg_group.group_id',
        onupdate="CASCADE", ondelete="CASCADE")),
    Column('permission_id', Integer, ForeignKey('tg_permission.permission_id',
        onupdate="CASCADE", ondelete="CASCADE"))
)

# This is the association table for the many-to-many relationship between
# groups and members - this is, the memberships. It's required by repoze.what.
user_group_table = Table('tg_user_group', metadata,
    Column('user_id', Integer, ForeignKey('tg_user.user_id',
        onupdate="CASCADE", ondelete="CASCADE")),
    Column('group_id', Integer, ForeignKey('tg_group.group_id',
        onupdate="CASCADE", ondelete="CASCADE"))
)


#{ The auth* model itself


class Group(DeclarativeBase):
    """
    Group definition for :mod:`repoze.what`.
    
    Only the ``group_name`` column is required by :mod:`repoze.what`.
    
    """
    
    __tablename__ = 'tg_group'
    
    #{ Columns
    
    group_id = Column(Integer, autoincrement=True, primary_key=True)
    
    group_name = Column(Unicode(16), unique=True, nullable=False)
    
    display_name = Column(Unicode(255))
    
    created = Column(DateTime, default=datetime.now)
    
    #{ Relations
    
    users = relation('User', secondary=user_group_table, backref='groups')
    
    #{ Special methods
    
    def __repr__(self):
        return '<Group: name=%s>' % self.group_name
    
    def __unicode__(self):
        return self.group_name
    
    #}


# The 'info' argument we're passing to the email_address and password columns
# contain metadata that Rum (http://python-rum.org/) can use generate an
# admin interface for your models.
class User(DeclarativeBase):
    """
    User definition.
    
    This is the user definition used by :mod:`repoze.who`, which requires at
    least the ``user_name`` column.
    
    """
    __tablename__ = 'tg_user'
    
    #{ Columns

    user_id = Column(Integer, autoincrement=True, primary_key=True)
    
    user_name = Column(Unicode(16), unique=True, nullable=False)
    
    email_address = Column(Unicode(255), unique=True, nullable=False,
                           info={'rum': {'field':'Email'}})
    
    display_name = Column(Unicode(255))
    
    _password = Column('password', Unicode(80),
                       info={'rum': {'field':'Password'}})
    
    created = Column(DateTime, default=datetime.now)
    
    #{ Special methods

    def __repr__(self):
        return '<User: email="%s", display name="%s">' % (
                self.email_address, self.display_name)

    def __unicode__(self):
        return self.display_name or self.user_name
    
    #{ Getters and setters

    @property
    def permissions(self):
        """Return a set of strings for the permissions granted."""
        perms = set()
        for g in self.groups:
            perms = perms | set(g.permissions)
        return perms

    @classmethod
    def by_email_address(cls, email):
        """Return the user object whose email address is ``email``."""
        return DBSession.query(cls).filter(cls.email_address==email).first()

    @classmethod
    def by_user_name(cls, username):
        """Return the user object whose user name is ``username``."""
        return DBSession.query(cls).filter(cls.user_name==username).first()

    def _set_password(self, password):
        """Hash ``password`` on the fly and store its hashed version."""
        hashed_password = password
        
        if isinstance(password, unicode):
            password_8bit = password.encode('UTF-8')
        else:
            password_8bit = password

        salt = sha1()
        salt.update(os.urandom(60))
        hash = sha1()
        hash.update(password_8bit + salt.hexdigest())
        hashed_password = salt.hexdigest() + hash.hexdigest()

        # Make sure the hashed password is an UTF-8 object at the end of the
        # process because SQLAlchemy _wants_ a unicode object for Unicode
        # columns
        if not isinstance(hashed_password, unicode):
            hashed_password = hashed_password.decode('UTF-8')

        self._password = hashed_password

    def _get_password(self):
        """Return the hashed version of the password."""
        return self._password

    password = synonym('_password', descriptor=property(_get_password,
                                                        _set_password))
    
    #}
    
    def validate_password(self, password):
        """
        Check the password against existing credentials.
        
        :param password: the password that was provided by the user to
            try and authenticate. This is the clear text version that we will
            need to match against the hashed one in the database.
        :type password: unicode object.
        :return: Whether the password is valid.
        :rtype: bool

        """
        hashed_pass = sha1()
        hashed_pass.update(password + self.password[:40])
        return self.password[40:] == hashed_pass.hexdigest()
       


class Permission(DeclarativeBase):
    """
    Permission definition for :mod:`repoze.what`.
    
    Only the ``permission_name`` column is required by :mod:`repoze.what`.
    
    """
    
    __tablename__ = 'tg_permission'
    
    #{ Columns

    permission_id = Column(Integer, autoincrement=True, primary_key=True)
    
    permission_name = Column(Unicode(16), unique=True, nullable=False)
    
    description = Column(Unicode(255))
    
    #{ Relations
    
    groups = relation(Group, secondary=group_permission_table,
                      backref='permissions')
    
    #{ Special methods
    
    def __repr__(self):
        return '<Permission: name=%s>' % self.permission_name

    def __unicode__(self):
        return self.permission_name
    
    #}


#}
